/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"fmt"
	"os"
	"strings"

	"github.com/spf13/cobra"
	"github.com/spf13/pflag"
	rcmigrate "k8s.libre.sh/rcmigrate/pkg"

	homedir "github.com/mitchellh/go-homedir"
	"github.com/spf13/viper"
)

var cfgFile string
var home = os.Getenv("HOME")

// initConfig reads in config file and ENV variables if set.
func initConfig(cmd *cobra.Command, config *rcmigrate.Config) error {
	v := viper.New()

	if cfgFile != "" {
		// Use config file from the flag.
		v.SetConfigFile(cfgFile)
	} else {
		// Find home directory.
		home, err := homedir.Dir()
		if err != nil {
			return err
		}

		// Search config in home directory with name ".keycloak-cli" (without extension).
		v.AddConfigPath(home)
		v.SetConfigName(".config")
	}

	//	v.AutomaticEnv() // read in environment variables that match

	// If a config file is found, read it in.
	err := v.ReadInConfig()
	if err == nil {
		fmt.Println("Using config file:", v.ConfigFileUsed())
	} else {
		return err
		// TODO check parse errors
	}
	// When we bind flags to environment variables expect that the
	// environment variables are prefixed, e.g. a flag like --number
	// binds to an environment variable STING_NUMBER. This helps
	// avoid conflicts.
	// v.SetEnvPrefix(envPrefix)

	// Bind to environment variables
	// Works great for simple config names, but needs help for names
	// like --favorite-color which we fix in the bindFlags function
	v.AutomaticEnv()

	// fmt.Println(v.AllKeys())

	// Bind the current command's flags to viper
	bindFlags(cmd, v)

	err = v.Unmarshal(config)
	if err != nil {
		fmt.Println(err)
		return err
	}

	fmt.Println("ffffffffffffff")
	fmt.Println(config)
	fmt.Println("ffffffffffffff")

	/* 	err = v.UnmarshalKey("nextcloudClient", config.NextcloudClient)
	   	if err != nil {
	   		return err
	   	}
	*/

	//keycloakConfig := v.Sub("cache.cache1")

	return nil
}

func LoadConfig(cmd *cobra.Command) (*rcmigrate.Config, error) {
	// all the other loading code

	config := new(rcmigrate.Config)
	if err := viper.Unmarshal(config); err != nil {
		return nil, err
	}

	// all the error signatures above had to change to nil, err
	return config, nil
}

// Bind each cobra flag to its associated viper configuration (config file and environment variable)
func bindFlags(cmd *cobra.Command, v *viper.Viper) {
	cmd.Flags().VisitAll(func(f *pflag.Flag) {
		// Environment variables can't have dashes in them, so bind them to their equivalent
		// keys with underscores, e.g. --favorite-color to STING_FAVORITE_COLOR
		if strings.Contains(f.Name, "-") {
			envVarSuffix := strings.ToUpper(strings.ReplaceAll(f.Name, "-", "_"))
			// v.BindEnv(f.Name, fmt.Sprintf("%s_%s", envPrefix, envVarSuffix))
			err := v.BindEnv(envVarSuffix)
			if err != nil {
				fmt.Println(err)
			}
		}

		// Apply the viper config value to the flag when the flag is not set and viper has a value
		if !f.Changed && v.IsSet(f.Name) {
			val := v.Get(f.Name)
			cmd.Flags().Set(f.Name, fmt.Sprintf("%v", val))
		}
	})
}
