/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"fmt"
	"os"

	"github.com/aws/aws-sdk-go/aws"
	"github.com/spf13/cobra"
	"go.mongodb.org/mongo-driver/mongo/options"
	rcmigrate "k8s.libre.sh/rcmigrate/pkg"
)

// rootCmd represents the base command when called without any subcommands
func NewRootCommand() *cobra.Command {
	config := &rcmigrate.Config{}
	mongo := &rcmigrate.Mongo{}
	mongo.ClientOptions = &options.ClientOptions{}
	// mongoOptions := &options.ClientOptions{}
	s3Config := &rcmigrate.S3{}
	s3Config.Config = &aws.Config{}

	config.S3 = s3Config
	config.Mongo = mongo

	rootCmd := &cobra.Command{
		Use:   "rcmigrate",
		Short: "A brief description of your application",
		Long:  "",
		PersistentPreRunE: func(cmd *cobra.Command, args []string) error {
			// return initConfig(cmd, config)
			err := initConfig(cmd, config)
			if err != nil {
				fmt.Println(err)
				return err
			}
			fmt.Println("BBBBBBBBBBBBBB")
			fmt.Println(config)
			fmt.Println(config.Mongo)
			fmt.Println(config.Mongo.ClientOptions)
			fmt.Println(config.S3.Config)
			fmt.Println("BBBBBBBBBBBBBB")

			return nil
		},
		RunE: func(cmd *cobra.Command, args []string) error {
			fmt.Println(config)
			fmt.Println(config.RocketChatURL)
			return nil
		},
	}

	fmt.Println("XXXXXXXXXXXXX")
	fmt.Println(config)
	fmt.Println("XXXXXXXXXXXXXXX")

	rootCmd.PersistentFlags().StringVar(&cfgFile, "config", "", "config file (default is $HOME/..yaml)")
	rootCmd.Flags().BoolP("toggle", "t", false, "Help message for toggle")

	rootCmd.PersistentFlags().StringSliceVarP(&config.Collections, "collections", "c", []string{"rocketchat_uploads", "rocketchat_avatars"}, "MongoDB Collections to migrate")

	rootCmd.AddCommand(NewRunCommand(config))
	rootCmd.AddCommand(NewMoveFilesCommand(config))
	rootCmd.AddCommand(NewUpdateDBCommand(config))
	rootCmd.AddCommand(NewDeleteFilesCommand(config))

	return rootCmd
}

func NewRunCommand(config *rcmigrate.Config) *cobra.Command {

	rootCmd := &cobra.Command{
		Use:   "run",
		Short: "A brief description of your application",
		Long:  "",
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.TODO()

			c, err := rcmigrate.NewClient(config)
			if err != nil {
				return err
			}

			tasks := rcmigrate.NewTasks("run")

			for _, coll := range config.Collections {
				fmt.Println(coll)
				if err := rcmigrate.Pipeline(ctx, c, coll, tasks); err != nil {
					return err
				}
			}

			return nil
		},
	}
	rootCmd.PersistentFlags().StringSliceVarP(&config.Collections, "collections", "c", []string{"rocketchat_uploads", "rocketchat_avatars"}, "MongoDB Collections to migrate")

	return rootCmd
}

func NewMoveFilesCommand(config *rcmigrate.Config) *cobra.Command {

	rootCmd := &cobra.Command{
		Use:   "move",
		Short: "A brief description of your application",
		Long:  "",
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.TODO()

			c, err := rcmigrate.NewClient(config)
			if err != nil {
				return err
			}

			tasks := rcmigrate.NewTasks("move")

			for _, coll := range config.Collections {
				if err := rcmigrate.Pipeline(ctx, c, coll, tasks); err != nil {
					return err
				}
			}

			return nil
		},
	}

	return rootCmd
}

func NewUpdateDBCommand(config *rcmigrate.Config) *cobra.Command {
	rootCmd := &cobra.Command{
		Use:   "updateDB",
		Short: "A brief description of your application",
		Long:  "",
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.TODO()

			c, err := rcmigrate.NewClient(config)
			if err != nil {
				return err
			}

			tasks := rcmigrate.NewTasks("updateDB")

			for _, coll := range config.Collections {
				if err := rcmigrate.Pipeline(ctx, c, coll, tasks); err != nil {
					return err
				}
			}

			return nil
		},
	}

	return rootCmd
}

func NewDeleteFilesCommand(config *rcmigrate.Config) *cobra.Command {
	rootCmd := &cobra.Command{
		Use:   "updateDB",
		Short: "A brief description of your application",
		Long:  "",
		RunE: func(cmd *cobra.Command, args []string) error {
			ctx := context.TODO()

			c, err := rcmigrate.NewClient(config)
			if err != nil {
				return err
			}

			tasks := rcmigrate.NewTasks("delete")

			for _, coll := range config.Collections {
				if err := rcmigrate.Pipeline(ctx, c, coll, tasks); err != nil {
					return err
				}
			}
			return nil
		},
	}

	return rootCmd
}

// Execute adds all child commands to the root command and sets flags appropriately.
// This is called by main.main(). It only needs to happen once to the rootCmd.
func Execute() {
	rootCmd := NewRootCommand()

	if err := rootCmd.Execute(); err != nil {
		fmt.Fprintln(os.Stderr, err)
		os.Exit(-1)
	}
}
