package pkg

import (
	"fmt"
	"net/url"
	"strings"

	"github.com/aws/aws-sdk-go/service/s3/s3manager"
)

type Upload struct {
	ID     string `json:"id" bson:"_id"`
	Name   string `json:"name"`
	Size   int    `json:"size"`
	Type   string `json:"type"`
	RID    string `json:"rid"`
	UserID string `json:"userId"`
	Store  string `json:"store"`
	//	UpdatedAt  primitive.DateTime `json:"_updatedAt"`
	//	InstanceID string             `json:"instanceId"`
	//	Identify   Identify           `json:"identify"`
	Complete bool   `json:"complete"`
	ETAG     string `json:"etag"`
	Path     string `json:"path"`
	// TODO TOFIX error decoding key progress: IntDecodeValue can only truncate float64 to an integer type when truncation is enabled
	// Progress   int64              `json:"progress"`
	//	Token      string             `json:"token"`
	//	UploadedAt primitive.DateTime `json:"uploadedAt"`
	//	Uploading  bool               `json:"uploading"`
	URL string `json:"url"`
	//	TypeGroup  string             `json:"typeGroup"`
	///	Description string `json:"description"`
	Extension string `json:"extension"`
	AmazonS3  `json:"AmazonS3"`
}

type AmazonS3 struct {
	Path string `json:"path"`
}

func NewS3UploadInput(u *Upload, bucket string) *s3manager.UploadInput {
	contentDisp := fmt.Sprintf("inline; filename=\"%s\"", url.PathEscape(u.Name))
	//	if len(u.AmazonS3.Path) > 0 {
	input := &s3manager.UploadInput{
		Bucket:             &bucket,
		Key:                &u.AmazonS3.Path,
		ContentType:        &u.Type,
		ContentDisposition: &contentDisp,
	}

	return input
}

func after(value string, a string) string {
	// Get substring after a string.
	pos := strings.LastIndex(value, a)
	if pos == -1 {
		return ""
	}
	adjustedPos := pos + len(a)
	if adjustedPos >= len(value) {
		return ""
	}
	// TODO TOFIX
	return value[adjustedPos:len(value)]
}

func before(value string, a string) string {
	// Get substring before a string.
	pos := strings.Index(value, a)
	if pos == -1 {
		return ""
	}
	return value[0:pos]
}

func (u *Upload) MutateToS3(rcURL, uid string) {
	uType := after(u.Store, ":")
	u.Store = fmt.Sprintf("AmazonS3:%s", uType)

	switch uType {
	case "Uploads":
		u.Path = fmt.Sprintf("/ufs/%s/%s/%s", u.ID, u.Store, url.PathEscape(u.Name))
		u.URL = fmt.Sprintf("%s/ufs/%s/%s/%s", rcURL, u.Store, u.ID, url.PathEscape(u.Name))
		u.AmazonS3.Path = fmt.Sprintf("%s/%s/%s/%s/%s", uid, strings.ToLower(uType), u.RID, u.UserID, u.ID)
	case "Avatars":
		u.URL = fmt.Sprintf("%s/ufs/%s/%s/undefined", rcURL, u.Store, u.ID)
		u.Path = fmt.Sprintf("/ufs/%s/%s/undefined", u.Store, u.ID)
		u.AmazonS3.Path = fmt.Sprintf("%s/%s/%s", uid, strings.ToLower(uType), u.ID)
	}
}

// TODO rcURL & uid
func (u *Upload) SetStore(rcURL, uid, store string) {
	uType := after(u.Store, ":")
	u.Store = fmt.Sprintf("%s:%s", store, uType)

	switch uType {
	case "Uploads":
		u.Path = fmt.Sprintf("/ufs/%s/%s/%s", u.ID, u.Store, url.PathEscape(u.Name))
		u.AmazonS3.Path = fmt.Sprintf("%s/%s/%s/%s/%s", uid, strings.ToLower(uType), u.RID, u.UserID, u.ID)
		u.URL = fmt.Sprintf("%s/ufs/%s/%s/%s", rcURL, u.Store, u.ID, url.PathEscape(u.Name))
	case "Avatars":
		u.Path = fmt.Sprintf("/ufs/%s/%s/undefined", u.Store, u.ID)
		u.AmazonS3.Path = fmt.Sprintf("%s/%s/%s", uid, strings.ToLower(uType), u.ID)
		u.URL = fmt.Sprintf("%s/ufs/%s/%s/undefined", rcURL, u.Store, u.ID)
	}
}

/* type Identify struct {
	Format string `json:"format"`
	Size   Size   `json:"size"`
}

type Size struct {
	Width  int `json:"width"`
	Height int `json:"height"`
} */
